"""Plugin-specific logic.
"""

import unittest
import json
import os.path

# 3rd Party imports
from jsonschema import validate


class TestPluginSyntax(unittest.TestCase):
    """Validates all plugins in `api/plugins/` using our JSON Schema.
    """
    # Get file paths relative to this file's known location
    schema_path = os.path.join(os.path.dirname(__file__), "../schemas/plugin-schema.json")
    plugins_dirs = [os.path.join(os.path.dirname(__file__), "../plugins/deployed"),
                    os.path.join(os.path.dirname(__file__), "../plugins/not_deployed") ]
    # Class variable holding the current JSON Schema. See setUpClass() for more info
    schema = None

    @classmethod
    def setUpClass(cls):
        """Open and read in the current plugin JSON Schema from disk for us during testing.
        """
        with open(cls.schema_path, mode='r') as f:
            cls.schema = json.load(f)
        cls.verificationFiles = []
        cls.errors = {}

    def tearDown(self):
        if len(self.errors) > 0:
            for file_path, err in self.errors.items():
                print("-"*70)
                print(f"Error validating plugin schema: ${file_path}")
                print(err)

            print("="*70)
            print ("Failed to validate plugins:")
            print("\n\t".join(self.verificationFiles))
            self.assertIsNone(1)

    def _test_plugin(self, plugin, path):
        """Test for single plugin file's content

        Args:
            plugin (dict): Content from plugin file
            path (str): Path to plugin file
        """
        try:
            self.assertIsNone(validate(instance=plugin, schema=self.schema))
        except Exception as e:
            self.errors[path] = e
            self.verificationFiles.append(path)

    def test_isvalid(self):
        """For each JSON file in the plugins directory, validate it against our schema. If an
        exception is raised, the plugin file is NOT valid.
        """
        for plugins_dir in self.plugins_dirs:
            for path in os.listdir(plugins_dir):
                path = os.path.join(plugins_dir, path)
                if os.path.isfile(path) and path.endswith('.json'):
                    with open(path, mode='r') as f:
                        try:
                            self._test_plugin(json.load(f), path)
                        except Exception as e:
                            print("Failed to load:", path, e)
                            self.errors[path] = e
                            self.verificationFiles.append(path)
